@@ -30,6 +30,10 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n-lib.h>
 
+#ifdef HAVE_MOBLIN
+#include <nbtk/nbtk-gtk.h>
+#endif
+
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-account.h>
 
@@ -62,7 +66,6 @@ typedef struct {
   gboolean simple;
 
   gboolean contains_pending_changes;
-  gboolean original_enabled_checkbox_value;
 
   /* An EmpathyAccountWidget can be used to either create an account or
    * modify it. When we are creating an account, this member is set to TRUE */
@@ -617,8 +620,14 @@ account_widget_applied_cb (GObject *source_object,
         {
           gboolean enabled_checked;
 
-          enabled_checked = gtk_toggle_button_get_active (
-              GTK_TOGGLE_BUTTON (priv->enabled_checkbox));
+          enabled_checked =
+#ifndef HAVE_MOBLIN
+            gtk_toggle_button_get_active (
+                GTK_TOGGLE_BUTTON (priv->enabled_checkbox));
+#else
+            nbtk_gtk_light_switch_get_active (
+                NBTK_GTK_LIGHT_SWITCH (priv->enabled_checkbox));
+#endif
 
           if (empathy_account_is_enabled (account) && enabled_checked)
             {
@@ -627,13 +636,6 @@ account_widget_applied_cb (GObject *source_object,
                * information entered by the user is validated on the server. */
               empathy_account_reconnect_async (account, NULL, NULL);
             }
-          else
-            {
-              /* The account is disabled so we enable it according to the value
-               * of the "Enabled" checkbox */
-              empathy_account_set_enabled_async (account, enabled_checked,
-                  NULL, NULL);
-            }
         }
     }
 
@@ -1055,17 +1057,39 @@ empathy_account_widget_enabled_cb (EmpathyAccount *account,
 
   if (priv->enabled_checkbox != NULL)
     {
+#ifndef HAVE_MOBLIN
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->enabled_checkbox),
           enabled);
+#else
+      nbtk_gtk_light_switch_set_active (
+          NBTK_GTK_LIGHT_SWITCH (priv->enabled_checkbox),
+          enabled);
+#endif /* HAVE_MOBLIN */
     }
 }
 
 static void
+#ifndef HAVE_MOBLIN
 account_widget_enabled_released_cb (GtkToggleButton *toggle_button,
     gpointer user_data)
+#else
+account_widget_switch_flipped_cb (NbtkGtkLightSwitch *sw,
+    gboolean state,
+    gpointer user_data)
+#endif /* HAVE_MOBLIN */
 {
-  account_widget_handle_control_buttons_sensitivity (
-      EMPATHY_ACCOUNT_WIDGET (user_data));
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (user_data);
+  EmpathyAccount *account;
+#ifndef HAVE_MOBLIN
+  gboolean state;
+
+  state = gtk_toggle_button_get_active (toggle_button);
+#endif
+
+  account = empathy_account_settings_get_account (priv->settings);
+
+  /* Enable the account according to the value of the "Enabled" checkbox */
+  empathy_account_set_enabled_async (account, state, NULL, NULL);
 }
 
 static void
@@ -1221,6 +1245,13 @@ do_constructed (GObject *obj)
       priv->apply_button = gtk_button_new_from_stock (
         priv->creating_account ? GTK_STOCK_CONNECT : GTK_STOCK_APPLY);
 
+#ifdef HAVE_MOBLIN
+      if (priv->creating_account)
+	/* Translators: this is used only when built on a moblin platform */
+	gtk_button_set_label (GTK_BUTTON (priv->apply_button),
+            _("L_og in"));
+#endif
+
       gtk_box_pack_end (GTK_BOX (hbox), priv->apply_button, TRUE,
           TRUE, 3);
       gtk_box_pack_end (GTK_BOX (hbox), priv->cancel_button, TRUE,
@@ -1258,15 +1289,32 @@ do_constructed (GObject *obj)
   /* handle the "Enabled" checkbox. We only add it when modifying an account */
   if (!priv->creating_account && priv->table_common_settings != NULL)
     {
+#ifdef HAVE_MOBLIN
+      GtkWidget *w;
+#endif
       guint nb_rows, nb_columns;
+      gboolean is_enabled;
 
+      is_enabled = empathy_account_is_enabled (account);
+
+#ifndef HAVE_MOBLIN
       priv->enabled_checkbox =
           gtk_check_button_new_with_label (_("Enabled"));
-      priv->original_enabled_checkbox_value =
-          empathy_account_is_enabled (account);
+
       gtk_toggle_button_set_active (
-          GTK_TOGGLE_BUTTON (priv->enabled_checkbox),
-          priv->original_enabled_checkbox_value);
+          GTK_TOGGLE_BUTTON (priv->enabled_checkbox), is_enabled);
+#else
+      /* Translators: this is used only when built on a moblin platform */
+      w = gtk_label_new (_("Account:"));
+      gtk_misc_set_alignment (GTK_MISC (w), 0, 0.5);
+
+      priv->enabled_checkbox = nbtk_gtk_light_switch_new ();
+
+      nbtk_gtk_light_switch_set_active (
+          NBTK_GTK_LIGHT_SWITCH (priv->enabled_checkbox), is_enabled);
+
+      gtk_widget_show (w);
+#endif /* HAVE_MOBLIN */
 
       g_object_get (priv->table_common_settings, "n-rows", &nb_rows,
           "n-columns", &nb_columns, NULL);
@@ -1274,14 +1322,31 @@ do_constructed (GObject *obj)
       gtk_table_resize (GTK_TABLE (priv->table_common_settings), ++nb_rows,
           nb_columns);
 
+#ifndef HAVE_MOBLIN
       gtk_table_attach (GTK_TABLE (priv->table_common_settings),
-          priv->enabled_checkbox, 0, nb_columns, nb_rows - 1, nb_rows,
+          priv->enabled_checkbox,
+          0, nb_columns, nb_rows - 1, nb_rows,
           GTK_EXPAND | GTK_FILL, 0, 0, 0);
+#else
+      gtk_table_attach (GTK_TABLE (priv->table_common_settings),
+          w,
+          0, 1, nb_rows - 1, nb_rows,
+          GTK_FILL, 0, 0, 0);
+      gtk_table_attach (GTK_TABLE (priv->table_common_settings),
+          priv->enabled_checkbox,
+          1, nb_columns, nb_rows - 1, nb_rows,
+          GTK_EXPAND | GTK_FILL, 0, 0, 0);
+#endif /* HAVE_MOBLIN */
 
       gtk_widget_show (priv->enabled_checkbox);
 
+#ifndef HAVE_MOBLIN
       g_signal_connect (G_OBJECT (priv->enabled_checkbox), "released",
           G_CALLBACK (account_widget_enabled_released_cb), self);
+#else
+      g_signal_connect (G_OBJECT (priv->enabled_checkbox), "switch-flipped",
+          G_CALLBACK (account_widget_switch_flipped_cb), self);
+#endif /* HAVE_MOBLIN */
     }
 
   /* hook up to widget destruction to unref ourselves */
@@ -1421,8 +1486,6 @@ empathy_account_widget_discard_pending_changes
   EmpathyAccountWidgetPriv *priv = GET_PRIV (widget);
 
   empathy_account_settings_discard_changes (priv->settings);
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->enabled_checkbox),
-      priv->original_enabled_checkbox_value);
   priv->contains_pending_changes = FALSE;
 }
 
