@@ -48,8 +48,9 @@ struct _EmpathySmileyManagerClass {
 };
 
 typedef struct {
-	GdkPixbuf *pixbuf;
-	gchar     *str;
+	GdkPixbuf   *pixbuf;
+	gchar       *str;
+	const gchar *path;
 } EmpathySmiley;
 
 typedef void (*EmpathySmileyMenuFunc) (EmpathySmileyManager *manager,
@@ -63,10 +64,6 @@ void                  empathy_smiley_manager_add             (EmpathySmileyManag
 							      const gchar          *icon_name,
 							      const gchar          *first_str,
 							      ...);
-void                  empathy_smiley_manager_add_from_pixbuf (EmpathySmileyManager *manager,
-							      GdkPixbuf            *smiley,
-							      const gchar          *first_str,
-							      ...);
 GSList *              empathy_smiley_manager_get_all         (EmpathySmileyManager *manager);
 GSList *              empathy_smiley_manager_parse           (EmpathySmileyManager *manager,
 							      const gchar          *text);
