@@ -79,9 +79,6 @@ GList *                 empathy_account_manager_dup_accounts
                                 (EmpathyAccountManager *manager);
 GList *                 empathy_account_manager_dup_connections
                                 (EmpathyAccountManager *manager);
-void                    empathy_account_manager_remove (
-                                 EmpathyAccountManager *manager,
-                                 EmpathyAccount *account);
 
 void empathy_account_manager_request_global_presence (
   EmpathyAccountManager *manager,
