@@ -26,6 +26,8 @@
 #include <telepathy-glib/util.h>
 #include <telepathy-glib/gtypes.h>
 #include <telepathy-glib/dbus.h>
+#include <telepathy-glib/interfaces.h>
+
 #if HAVE_GEOCLUE
 #include <geoclue/geoclue-geocode.h>
 #endif
@@ -128,7 +130,7 @@ tp_contact_factory_set_aliases_cb (TpConnection *connection,
 }
 
 static void
-tp_contact_factory_set_location_cb (TpProxy *proxy,
+tp_contact_factory_set_location_cb (TpConnection *tp_conn,
 				    const GError *error,
 				    gpointer user_data,
 				    GObject *weak_object)
@@ -235,60 +237,51 @@ tp_contact_factory_avatar_maybe_update (EmpathyTpContactFactory *tp_factory,
 	return FALSE;
 }
 
-typedef struct {
-	EmpathyTpContactFactory *tp_factory;
-	GArray                  *handles;
-} TokensData;
-
 static void
-tp_contact_factory_avatar_tokens_foreach (gpointer key,
-					  gpointer value,
-					  gpointer user_data)
-{
-	TokensData  *data = user_data;
-	const gchar *token = value;
-	guint        handle = GPOINTER_TO_UINT (key);
-
-	if (!tp_contact_factory_avatar_maybe_update (data->tp_factory,
-						     handle, token)) {
-		g_array_append_val (data->handles, handle);
-	}
-}
-
-static void
-tp_contact_factory_got_known_avatar_tokens (EmpathyTpContactFactory *tp_factory,
-					    GHashTable              *tokens,
-					    const GError            *error)
+tp_contact_factory_got_known_avatar_tokens (TpConnection *connection,
+					    GHashTable   *tokens,
+					    const GError *error,
+					    gpointer      user_data,
+					    GObject      *weak_object)
 {
+	EmpathyTpContactFactory *tp_factory = EMPATHY_TP_CONTACT_FACTORY (weak_object);
 	EmpathyTpContactFactoryPriv *priv = GET_PRIV (tp_factory);
-	TokensData data;
+	GArray *handles;
+	GHashTableIter iter;
+	gpointer key, value;
 
 	if (error) {
 		DEBUG ("Error: %s", error->message);
 		return;
 	}
 
-	data.tp_factory = tp_factory;
-	data.handles = g_array_new (FALSE, FALSE, sizeof (guint));
-	g_hash_table_foreach (tokens,
-			      tp_contact_factory_avatar_tokens_foreach,
-			      &data);
+	handles = g_array_new (FALSE, FALSE, sizeof (guint));
+
+	g_hash_table_iter_init (&iter, tokens);
+	while (g_hash_table_iter_next (&iter, &key, &value)) {
+		guint handle = GPOINTER_TO_UINT (key);
+		const gchar *token = value;
+
+		if (!tp_contact_factory_avatar_maybe_update (tp_factory,
+							     handle, token)) {
+			g_array_append_val (handles, handle);
+		}
+	}
 
 	DEBUG ("Got %d tokens, need to request %d avatars",
-		g_hash_table_size (tokens), data.handles->len);
+		g_hash_table_size (tokens), handles->len);
 
 	/* Request needed avatars */
-	if (data.handles->len > 0) {
+	if (handles->len > 0) {
 		tp_cli_connection_interface_avatars_call_request_avatars (priv->connection,
 									  -1,
-									  data.handles,
+									  handles,
 									  tp_contact_factory_request_avatars_cb,
 									  NULL, NULL,
 									  G_OBJECT (tp_factory));
 	}
 
-	g_array_free (data.handles, TRUE);
-	g_hash_table_destroy (tokens);
+	g_array_free (handles, TRUE);
 }
 
 static void
@@ -358,12 +351,17 @@ tp_contact_factory_update_capabilities (EmpathyTpContactFactory *tp_factory,
 }
 
 static void
-tp_contact_factory_got_capabilities (EmpathyTpContactFactory *tp_factory,
-				     GPtrArray *capabilities,
-				     const GError    *error)
+tp_contact_factory_got_capabilities (TpConnection    *connection,
+				     const GPtrArray *capabilities,
+				     const GError    *error,
+				     gpointer         user_data,
+				     GObject         *weak_object)
 {
+	EmpathyTpContactFactory *tp_factory;
 	guint i;
 
+	tp_factory = EMPATHY_TP_CONTACT_FACTORY (weak_object);
+
 	if (error) {
 		DEBUG ("Error: %s", error->message);
 		/* FIXME Should set the capabilities of the contacts for which this request
@@ -389,11 +387,7 @@ tp_contact_factory_got_capabilities (EmpathyTpContactFactory *tp_factory,
 							channel_type,
 							generic,
 							specific);
-
-		g_value_array_free (values);
 	}
-
-	g_ptr_array_free (capabilities, TRUE);
 }
 
 #if HAVE_GEOCLUE
@@ -568,7 +562,7 @@ tp_contact_factory_update_location (EmpathyTpContactFactory *tp_factory,
 }
 
 static void
-tp_contact_factory_got_locations (TpProxy                 *tp_proxy,
+tp_contact_factory_got_locations (TpConnection                 *tp_conn,
 				  GHashTable              *locations,
 				  const GError            *error,
 				  gpointer                 user_data,
@@ -624,7 +618,7 @@ tp_contact_factory_capabilities_changed_cb (TpConnection    *connection,
 }
 
 static void
-tp_contact_factory_location_updated_cb (TpProxy      *proxy,
+tp_contact_factory_location_updated_cb (TpConnection      *tp_conn,
 					guint         handle,
 					GHashTable   *location,
 					gpointer      user_data,
@@ -738,9 +732,6 @@ tp_contact_factory_add_contact (EmpathyTpContactFactory *tp_factory,
 	TpHandle self_handle;
 	TpHandle handle;
 	GArray handles = {(gchar *) &handle, 1};
-	GHashTable *tokens;
-	GPtrArray *capabilities;
-	GError *error = NULL;
 	EmpathyCapabilities caps;
 
 	/* Keep a weak ref to that contact */
@@ -776,27 +767,25 @@ tp_contact_factory_add_contact (EmpathyTpContactFactory *tp_factory,
 	empathy_contact_set_is_user (contact, self_handle == handle);
 
 	/* FIXME: This should be done by TpContact */
-	tp_cli_connection_interface_avatars_run_get_known_avatar_tokens (priv->connection,
-									 -1,
-									 &handles,
-									 &tokens,
-									 &error,
-									 NULL);
-	tp_contact_factory_got_known_avatar_tokens (tp_factory, tokens, error);
-	g_clear_error (&error);
+	if (tp_proxy_has_interface_by_id (priv->connection,
+			TP_IFACE_QUARK_CONNECTION_INTERFACE_AVATARS)) {
+		tp_cli_connection_interface_avatars_call_get_known_avatar_tokens (
+			priv->connection, -1, &handles,
+			tp_contact_factory_got_known_avatar_tokens, NULL, NULL,
+			G_OBJECT (tp_factory));
+	}
 
-	tp_cli_connection_interface_capabilities_run_get_capabilities (priv->connection,
-									-1,
-									&handles,
-									&capabilities,
-									&error,
-									NULL);
-	tp_contact_factory_got_capabilities (tp_factory, capabilities, error);
-	g_clear_error (&error);
+	if (tp_proxy_has_interface_by_id (priv->connection,
+			TP_IFACE_QUARK_CONNECTION_INTERFACE_CAPABILITIES)) {
+		tp_cli_connection_interface_capabilities_call_get_capabilities (
+			priv->connection, -1, &handles,
+			tp_contact_factory_got_capabilities, NULL, NULL,
+			G_OBJECT (tp_factory));
+	}
 
 	if (tp_proxy_has_interface_by_id (TP_PROXY (priv->connection),
-		EMP_IFACE_QUARK_CONNECTION_INTERFACE_LOCATION)) {
-		emp_cli_connection_interface_location_call_get_locations (TP_PROXY (priv->connection),
+		TP_IFACE_QUARK_CONNECTION_INTERFACE_LOCATION)) {
+		tp_cli_connection_interface_location_call_get_locations (priv->connection,
 									 -1,
 									 &handles,
 									 tp_contact_factory_got_locations,
@@ -1045,6 +1034,11 @@ empathy_tp_contact_factory_get_from_handles (EmpathyTpContactFactory *tp_factory
 	EmpathyTpContactFactoryPriv *priv = GET_PRIV (tp_factory);
 	GetContactsData *data;
 
+	if (n_handles == 0) {
+		callback (tp_factory, 0, NULL, 0, NULL, NULL, user_data, weak_object);
+		return;
+	}
+
 	g_return_if_fail (EMPATHY_IS_TP_CONTACT_FACTORY (tp_factory));
 	g_return_if_fail (handles != NULL);
 
@@ -1202,7 +1196,7 @@ empathy_tp_contact_factory_set_location (EmpathyTpContactFactory *tp_factory,
 
 	DEBUG ("Setting location");
 
-	emp_cli_connection_interface_location_call_set_location (TP_PROXY (priv->connection),
+	tp_cli_connection_interface_location_call_set_location (priv->connection,
 								 -1,
 								 location,
 								 tp_contact_factory_set_location_cb,
@@ -1316,7 +1310,7 @@ tp_contact_factory_constructor (GType                  type,
 										  NULL);
 
 
-	emp_cli_connection_interface_location_connect_to_location_updated (TP_PROXY (priv->connection),
+	tp_cli_connection_interface_location_connect_to_location_updated (priv->connection,
 									   tp_contact_factory_location_updated_cb,
 									   NULL, NULL,
 									   G_OBJECT (tp_factory),
