@@ -35,7 +35,6 @@
 #include <libnotify/notification.h>
 
 #include <telepathy-glib/util.h>
-#include <libmissioncontrol/mission-control.h>
 
 #include <libempathy/empathy-contact.h>
 #include <libempathy/empathy-message.h>
@@ -60,6 +59,11 @@
 #define DEBUG_FLAG EMPATHY_DEBUG_CHAT
 #include <libempathy/empathy-debug.h>
 
+typedef struct {
+	EmpathyChatWindow *window;
+	EmpathyChat *chat;
+} NotificationData;
+
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyChatWindow)
 typedef struct {
 	EmpathyChat *current_chat;
@@ -73,6 +77,7 @@ typedef struct {
 	GtkWidget   *dialog;
 	GtkWidget   *notebook;
 	NotifyNotification *notification;
+	NotificationData *notification_data;
 
 	/* Menu items. */
 	GtkUIManager *ui_manager;
@@ -108,6 +113,8 @@ static const GtkTargetEntry drag_types_dest[] = {
 	{ "GTK_NOTEBOOK_TAB", GTK_TARGET_SAME_APP, DND_DRAG_TYPE_TAB },
 };
 
+static void chat_window_update (EmpathyChatWindow *window);
+
 G_DEFINE_TYPE (EmpathyChatWindow, empathy_chat_window, G_TYPE_OBJECT);
 
 static void
@@ -289,6 +296,17 @@ chat_window_create_label (EmpathyChatWindow *window,
 	return hbox;
 }
 
+static void
+_submenu_notify_visible_changed_cb (GObject    *object,
+				    GParamSpec *pspec,
+				    gpointer    userdata)
+{
+	g_signal_handlers_disconnect_by_func (object,
+					      _submenu_notify_visible_changed_cb,
+					      userdata);
+	chat_window_update (EMPATHY_CHAT_WINDOW (userdata));
+}
+
 static void
 chat_window_update (EmpathyChatWindow *window)
 {
@@ -306,8 +324,7 @@ chat_window_update (EmpathyChatWindow *window)
 	gboolean               avatar_in_icon;
 	GtkWidget             *chat;
 	GtkWidget             *chat_close_button;
-	GtkWidget             *submenu;
-	GtkWidget             *menu;
+	GtkWidget             *menu, *submenu, *orig_submenu;
 
 	/* Get information */
 	page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
@@ -331,9 +348,17 @@ chat_window_update (EmpathyChatWindow *window)
 	/* Update Contact menu */
 	menu = gtk_ui_manager_get_widget (priv->ui_manager,
 		"/chats_menubar/menu_contact");
-	submenu = empathy_chat_get_contact_menu (priv->current_chat);
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu), submenu);
-	gtk_widget_show (menu);
+	orig_submenu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (menu));
+	if (orig_submenu == NULL || !GTK_WIDGET_VISIBLE (orig_submenu)) {
+		submenu = empathy_chat_get_contact_menu (priv->current_chat);
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu), submenu);
+		gtk_widget_show (menu);
+	} else {
+		empathy_signal_connect_weak (orig_submenu,
+					     "notify::visible",
+					     (GCallback)_submenu_notify_visible_changed_cb,
+					     G_OBJECT (window));
+	}
 
 	/* Update window title */
 	if (n_chats == 1) {
@@ -851,7 +876,7 @@ chat_window_help_contents_activate_cb (GtkAction         *action,
 {
 	EmpathyChatWindowPriv *priv = GET_PRIV (window);
 
-	empathy_url_show (priv->dialog, "ghelp:empathy?chat");
+	empathy_url_show (priv->dialog, "ghelp:empathy");
 }
 
 static void
@@ -911,10 +936,12 @@ chat_window_set_urgency_hint (EmpathyChatWindow *window,
 	gtk_window_set_urgency_hint (GTK_WINDOW (priv->dialog), urgent);
 }
 
-typedef struct {
-	EmpathyChatWindow *window;
-	EmpathyChat *chat;
-} NotificationData;
+static void
+free_notification_data (NotificationData *data)
+{
+	g_object_unref (data->chat);
+	g_slice_free (NotificationData, data);
+}
 
 static void
 chat_window_notification_closed_cb (NotifyNotification *notify,
@@ -932,8 +959,8 @@ chat_window_notification_closed_cb (NotifyNotification *notify,
 
 	g_object_unref (notify);
 	priv->notification = NULL;
-	g_object_unref (cb_data->chat);
-	g_slice_free (NotificationData, cb_data);
+	free_notification_data (cb_data);
+	priv->notification_data = NULL;
 }
 
 static void
@@ -946,7 +973,6 @@ chat_window_show_or_update_notification (EmpathyChatWindow *window,
 	char *escaped;
 	const char *body;
 	GdkPixbuf *pixbuf;
-	NotificationData *cb_data;
 	EmpathyChatWindowPriv *priv = GET_PRIV (window);
 	gboolean res;
 
@@ -960,37 +986,37 @@ chat_window_show_or_update_notification (EmpathyChatWindow *window,
 		}
 	}
 
-	cb_data = g_slice_new0 (NotificationData);
-	cb_data->chat = g_object_ref (chat);
-	cb_data->window = window;
-
 	sender = empathy_message_get_sender (message);
 	header = empathy_contact_get_name (sender);
 	body = empathy_message_get_body (message);
 	escaped = g_markup_escape_text (body, -1);
 
-	pixbuf = empathy_misc_get_pixbuf_for_notification (sender, EMPATHY_IMAGE_NEW_MESSAGE);
-
 	if (priv->notification != NULL) {
 		notify_notification_update (priv->notification,
 					    header, escaped, NULL);
-		/* if icon doesn't exist libnotify will crash */
-		if (pixbuf != NULL)
-			notify_notification_set_icon_from_pixbuf (priv->notification, pixbuf);
 	} else {
+		NotificationData *cb_data = cb_data = g_slice_new0 (NotificationData);
+
+		cb_data->chat = g_object_ref (chat);
+		cb_data->window = window;
+
+		priv->notification_data = cb_data;
 		priv->notification = notify_notification_new (header, escaped, NULL, NULL);
 		notify_notification_set_timeout (priv->notification, NOTIFY_EXPIRES_DEFAULT);
-		/* if icon doesn't exist libnotify will crash */
-		if (pixbuf != NULL)
-			notify_notification_set_icon_from_pixbuf (priv->notification, pixbuf);
 
 		g_signal_connect (priv->notification, "closed",
 				  G_CALLBACK (chat_window_notification_closed_cb), cb_data);
 	}
 
+	pixbuf = empathy_misc_get_pixbuf_for_notification (sender, EMPATHY_IMAGE_NEW_MESSAGE);
+
+	if (pixbuf != NULL) {
+		notify_notification_set_icon_from_pixbuf (priv->notification, pixbuf);
+		g_object_unref (pixbuf);
+	}
+
 	notify_notification_show (priv->notification, NULL);
 
-	g_object_unref (pixbuf);
 	g_free (escaped);
 }
 
@@ -1003,7 +1029,8 @@ chat_window_set_highlight_room_tab_label (EmpathyChat *chat)
 	if (!empathy_chat_is_room (chat))
 		return;
 
-	markup = g_markup_printf_escaped ("<span color=\"red\">%s</span>",
+	markup = g_markup_printf_escaped (
+		"<span color=\"red\" weight=\"bold\">%s</span>",
 		empathy_chat_get_name (chat));
 
 	widget = g_object_get_data (G_OBJECT (chat), "chat-window-tab-label");
@@ -1252,9 +1279,9 @@ chat_window_drag_data_received (GtkWidget        *widget,
 				EmpathyChatWindow *window)
 {
 	if (info == DND_DRAG_TYPE_CONTACT_ID) {
-		EmpathyChat           *chat;
+		EmpathyChat           *chat = NULL;
 		EmpathyChatWindow     *old_window;
-		EmpathyAccount        *account;
+		EmpathyAccount        *account = NULL;
 		EmpathyAccountManager *account_manager;
 		const gchar           *id;
 		gchar                **strv;
@@ -1266,11 +1293,21 @@ chat_window_drag_data_received (GtkWidget        *widget,
 
 		DEBUG ("DND contact from roster with id:'%s'", id);
 
-		strv = g_strsplit (id, "/", 2);
-		account_id = strv[0];
-		contact_id = strv[1];
-		account = empathy_account_manager_lookup (account_manager, account_id);
-		chat = empathy_chat_window_find_chat (account, contact_id);
+		strv = g_strsplit (id, ":", 2);
+		if (g_strv_length (strv) == 2) {
+			account_id = strv[0];
+			contact_id = strv[1];
+			account =
+				empathy_account_manager_get_account (account_manager, account_id);
+			if (account != NULL)
+				chat = empathy_chat_window_find_chat (account, contact_id);
+		}
+
+		if (account == NULL) {
+			g_strfreev (strv);
+			gtk_drag_finish (context, FALSE, FALSE, time);
+			return;
+		}
 
 		if (!chat) {
 			TpConnection *connection;
@@ -1282,11 +1319,9 @@ chat_window_drag_data_received (GtkWidget        *widget,
 					connection, contact_id, NULL, NULL);
 			}
 
-			g_object_unref (account);
 			g_strfreev (strv);
 			return;
 		}
-		g_object_unref (account);
 		g_object_unref (account_manager);
 		g_strfreev (strv);
 
@@ -1369,6 +1404,11 @@ chat_window_finalize (GObject *object)
 		notify_notification_close (priv->notification, NULL);
 		g_object_unref (priv->notification);
 		priv->notification = NULL;
+		if (priv->notification_data != NULL)
+			{
+				free_notification_data (priv->notification_data);
+				priv->notification_data = NULL;
+			}
 	}
 
 	chat_windows = g_list_remove (chat_windows, window);
@@ -1778,7 +1818,7 @@ empathy_chat_window_find_chat (EmpathyAccount   *account,
 
 			chat = ll->data;
 
-			if (empathy_account_equal (account, empathy_chat_get_account (chat)) &&
+			if (account == empathy_chat_get_account (chat) &&
 			    !tp_strdiff (id, empathy_chat_get_id (chat))) {
 				return chat;
 			}
