@@ -23,10 +23,16 @@
 
 #include <string.h>
 
+#include <glib.h>
+
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
+#include <glib/gi18n.h>
 
 #include <libnotify/notification.h>
+#include <libnotify/notify.h>
+
+#include <telepathy-glib/util.h>
 
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-idle.h>
@@ -67,6 +73,7 @@ typedef struct {
 	GtkAction           *show_window_item;
 	GtkAction           *new_message_item;
 	GtkAction           *status_item;
+	gboolean             notify_supports_actions;
 } EmpathyStatusIconPriv;
 
 G_DEFINE_TYPE (EmpathyStatusIcon, empathy_status_icon, G_TYPE_OBJECT);
@@ -123,6 +130,19 @@ notification_close_helper (EmpathyStatusIconPriv *priv)
 	}
 }
 
+#ifdef HAVE_MOBLIN
+static void
+notification_action_cb (NotifyNotification *notification,
+			gchar              *action,
+			EmpathyStatusIcon  *icon)
+{
+	EmpathyStatusIconPriv *priv = GET_PRIV (icon);
+
+	if (priv->event)
+		empathy_event_activate (priv->event);
+}
+#endif
+
 static void
 status_icon_update_notification (EmpathyStatusIcon *icon)
 {
@@ -136,30 +156,48 @@ status_icon_update_notification (EmpathyStatusIcon *icon)
 	}
 
 	if (priv->event) {
-		pixbuf = empathy_misc_get_pixbuf_for_notification (priv->event->contact,
-								   priv->event->icon_name);
+		gchar *message_esc = NULL;
+
+		if (priv->event->message != NULL)
+			message_esc = g_markup_escape_text (priv->event->message, -1);
 
 		if (priv->notification) {
 			notify_notification_update (priv->notification,
-						    priv->event->header, priv->event->message,
+						    priv->event->header, message_esc,
 						    NULL);
 		} else {
 			priv->notification = notify_notification_new_with_status_icon
-				(priv->event->header, priv->event->message, NULL, priv->icon);
+				(priv->event->header, message_esc, NULL, priv->icon);
 			notify_notification_set_timeout (priv->notification,
 							 NOTIFY_EXPIRES_DEFAULT);
 
+#ifdef HAVE_MOBLIN
+			if (priv->notify_supports_actions) {
+				notify_notification_add_action (priv->notification,
+					"respond",
+					_("Respond"),
+					(NotifyActionCallback) notification_action_cb,
+					icon,
+					NULL);
+			}
+#endif
+
 			g_signal_connect (priv->notification, "closed",
 					  G_CALLBACK (status_icon_notification_closed_cb), icon);
+		}
 
- 		}
-		/* if icon doesn't exist libnotify will crash */
-		if (pixbuf != NULL)
+		pixbuf = empathy_misc_get_pixbuf_for_notification (priv->event->contact,
+								   priv->event->icon_name);
+
+		if (pixbuf != NULL) {
 			notify_notification_set_icon_from_pixbuf (priv->notification,
 							  pixbuf);
+			g_object_unref (pixbuf);
+		}
+
 		notify_notification_show (priv->notification, NULL);
 
-		g_object_unref (pixbuf);
+		g_free (message_esc);
 	} else {
 		notification_close_helper (priv);
 	}
@@ -173,12 +211,12 @@ status_icon_update_tooltip (EmpathyStatusIcon *icon)
 
 	if (priv->event) {
 		if (priv->event->message != NULL)
-				tooltip = g_strdup_printf ("<i>%s</i>\n%s",
-							   priv->event->header,
-							   priv->event->message);
+				tooltip = g_markup_printf_escaped ("<i>%s</i>\n%s",
+								   priv->event->header,
+								   priv->event->message);
 		else
-				tooltip = g_strdup_printf ("<i>%s</i>",
-							   priv->event->header);
+				tooltip = g_markup_printf_escaped ("<i>%s</i>",
+								   priv->event->header);
 		gtk_status_icon_set_tooltip_markup (priv->icon, tooltip);
 	} else {
 		tooltip = g_strdup (empathy_idle_get_status (priv->idle));
@@ -305,24 +343,7 @@ status_icon_set_visibility (EmpathyStatusIcon *icon,
 	if (!visible) {
 		empathy_window_iconify (priv->window, priv->icon);
 	} else {
-		GList *accounts;
-		GList *l;
-		gboolean one_enabled = FALSE;
-
 		empathy_window_present (GTK_WINDOW (priv->window), TRUE);
-
-		/* Show the accounts dialog if there is no enabled accounts */
-		accounts = empathy_account_manager_dup_accounts (priv->account_manager);
-		for (l = accounts ; l != NULL ; l = g_list_next (l)) {
-			one_enabled = empathy_account_is_enabled (EMPATHY_ACCOUNT (l->data))
-				|| one_enabled;
-			g_object_unref (l->data);
-		}
-		g_list_free (accounts);
-		if (!one_enabled) {
-			DEBUG ("No enabled account, Showing account dialog");
-			empathy_accounts_dialog_show (GTK_WINDOW (priv->window), NULL);
-		}
 	}
 }
 
@@ -546,6 +567,7 @@ empathy_status_icon_init (EmpathyStatusIcon *icon)
 {
 	EmpathyStatusIconPriv *priv = G_TYPE_INSTANCE_GET_PRIVATE (icon,
 		EMPATHY_TYPE_STATUS_ICON, EmpathyStatusIconPriv);
+	GList *list, *l;
 
 	icon->priv = priv;
 	priv->icon = gtk_status_icon_new ();
@@ -584,6 +606,17 @@ empathy_status_icon_init (EmpathyStatusIcon *icon)
 	g_signal_connect (priv->icon, "popup-menu",
 			  G_CALLBACK (status_icon_popup_menu_cb),
 			  icon);
+
+	/* Query the notification daemon to check if it supports actions */
+	priv->notify_supports_actions = FALSE;
+	list = notify_get_server_caps ();
+	for (l = list; l != NULL; l = g_list_next (l)) {
+		gchar *caps = l->data;
+		if (!tp_strdiff (caps, "actions"))
+			priv->notify_supports_actions = TRUE;
+		g_free (caps);
+	}
+	g_list_free (list);
 }
 
 EmpathyStatusIcon *
